import { linter } from '@codemirror/next/lint';
export const slowUpdateLinter = (connection, {
  slowUpdateResult
}) => {
  let resolveDiagnostics;

  const mapDiagnostic = ({
    span: {
      start,
      length
    },
    message,
    severity
  }) => ({
    from: start,
    to: start + length,
    message,
    severity: severity
  });

  connection.on('message', message => {
    if (message.type !== 'slowUpdate') return;

    if (resolveDiagnostics) {
      const diagnostics = message.diagnostics.map(mapDiagnostic);
      diagnostics.sort((a, b) => {
        if (a.from > b.from) return 1;
        if (b.from > a.from) return -1;
        return 0;
      });
      resolveDiagnostics(diagnostics);
    }

    if (slowUpdateResult) slowUpdateResult(message);
  });
  return linter(async () => {
    const promise = new Promise(r => resolveDiagnostics = r);
    await connection.sendSlowUpdate();
    return promise;
  });
};